% Spectral_Sim.m
% Jason Wu
% 2/22/15
%
% This function calculates correlated epsilons for the simulation of ground
% motion values using the spectral formulation found in: 
% Fenton, GA. Simulation and Analysis of Random Fields. Ph. D. Dissertation, 
% Princeton University: New Jersey, 1990. 
% http://www.researchgate.net/publication/34099473_Simulation_and_analysis_of_random_fields
% 
% Note that a sites arranged in an evenly spaced grid is required for this 
% formulation.
% 
% The model variogram used can be found in:
% Jayaram, N. Probabilistic Seismic Lifeline Risk Assessment Using 
% Efficient Sampling and Data Reduction Techniques. Ph. D. Dissertation, 
% Stanford University: California, 2010. http://purl.stanford.edu/hp846fv1725
% 
% INPUTS (if input structure is not used):
%
% MasterFolder : string : path to the folder that will contain relavent
%       precomputations for these calculations
% scale : integer : Positive integer indicating the number of sinusoids
%       used for simulation. Increased scale may improve accuracy, but may
%       take more time.
%       scale = 1 = number of sinusoids equal to the number of sites in
%           each row or column in the site grid (the larger of the two).
%       scale = 2 = twice the number of sinusoids as scale = 1
%       scale = x = x times the number of sinusoids as scale = 1
% period: Scalar : Scalar indicating the period of the spectral
%       acceleration for which epsilons are being calculated. This
%       value should correspond with the following:
%       period = 1 if the intensity measure is PGV
%       period = 0 if the intensity measure is PGA
%       period = # if the intensity measure is SA, where # represents
%           the period corresponding to the SA
% nSitesX : Integer : Indicates the number of sites in the grid in the x
%       direction
% nSitesY : Integer : Indicates the number of sites in the gride in the y
%       direction
% spacing : Scalar : Indicates the spacing between sites in kilometers
% OverwriteFlag : logical : Flag to overwrite saved data for new simulations
%       OverwriteFlag = 0 = do not overwrite
%       OverwriteFlag = 1 = overwrite
%
% OUTPUTS:
%
% epsilons : Matrix : n x 1 matrix indicating the
%       realizations of the correlated epsilons for each n sites for
%       each earthquake scenario (numSimulations of them)
% 
% NOTE:
% The user must run Site_Creator.m and have the information saved to the
% save master folder prior to running Spectral_Sim.m

function epsilons = Spectral_Sim(SimInput)

if nargin == 0
    % User Input if input structure is not specified
    Identifier = '030414_SF_05km_large';
    BaseFolder = 'C:\Jason Files\Baker Research 13\';
    MasterFolder = [BaseFolder 'MasterFolders\MasterFolder_' Identifier '\'];

    scale = 1;
    period = 1;
    nSitesX = 25;
    nSitesY = 25;
    spacing = 0.5;
    
    OverwriteFlag = 1;
    
else
    MasterFolder = SimInput.MasterFolder;
    period = SimInput.period;
    OverwriteFlag = SimInput.OverwriteFlag;
    scale = SimInput.SpectralParams.scale;
    spacing = SimInput.SpectralParams.spacing;
    
    load([MasterFolder 'siteInfo.mat'])
%   nSitesX = number of sites along X direction
%   nSitesY = number of sites along Y direction
%   GridSpacing = spacing in between sites in km

    
end

% Preparations
numSitesPerRow = max(nSitesX,nSitesY);
numSitesPerRow = numSitesPerRow*scale;

paramFilepath = [MasterFolder 'Params_Spectral.mat'];



if ~exist(paramFilepath,'file') || OverwriteFlag == 1

    % Calculate small distance matrix
    xcoord = [0:numSitesPerRow-1]*spacing;
    ycoord = xcoord;
    [X1,Y1] = meshgrid(xcoord,ycoord);
    h = sqrt(X1.^2+Y1.^2);
    
    if period < 1
        b = 40.7 - 15*period; % For short periods, clustering in Vs30
        %     b = 8.5 + 17.2*period; % For short periods, no clustering in Vs30
    else
        b = 22.0 + 3.7*period; % For long periods, period >= 1
    end
    
    cov = exp(-3*h/b);
    
    cov_mat = zeros(numSitesPerRow*2,numSitesPerRow*2);
    cov_mat(1:numSitesPerRow,1:numSitesPerRow) = cov;
    cov_mat(1:numSitesPerRow,numSitesPerRow+1:end) = fliplr(cov);
    cov_mat(numSitesPerRow+1:end,1:numSitesPerRow) = flipud(cov);
    cov_mat(numSitesPerRow+1:end,numSitesPerRow+1:end) = rot90(cov,2);
    
    Fk = fft2(cov_mat);
    Fk = 8*Fk;
    
    cov_A = zeros(size(cov_mat));
    %% Pending
    cov_A(1,1) = numSitesPerRow^2*Fk(1,1);
    cov_A(1,numSitesPerRow+1) = numSitesPerRow^2*Fk(1,numSitesPerRow+1);
    cov_A(numSitesPerRow+1,1) = numSitesPerRow^2*Fk(numSitesPerRow+1,1);
    cov_A(numSitesPerRow+1,numSitesPerRow+1) = numSitesPerRow^2*Fk(numSitesPerRow+1,numSitesPerRow+1);
    for j = [2:numSitesPerRow numSitesPerRow+2:2*numSitesPerRow]
        for k = [1 numSitesPerRow+1]
            cov_A(k,j) = (Fk(k,j) + Fk(k,2*numSitesPerRow-j+2))/4*numSitesPerRow^2;
        end
    end
    
    for k = [2:numSitesPerRow numSitesPerRow+2:2*numSitesPerRow]
        for j = [1 numSitesPerRow+1]
            cov_A(k,j) = (Fk(k,j) + Fk(2*numSitesPerRow-k+2,j))/4*numSitesPerRow^2;
        end
    end
    
    for k = [2:numSitesPerRow numSitesPerRow+2:2*numSitesPerRow]
        for j = [2:numSitesPerRow numSitesPerRow+2:2*numSitesPerRow]
            cov_A(k,j) = (Fk(k,j) + Fk(k,2*numSitesPerRow-j+2) + Fk(2*numSitesPerRow-k+2,j) + Fk(2*numSitesPerRow-k+2,2*numSitesPerRow-j+2))*numSitesPerRow^2/8;
        end
    end
    
    cov_B = cov_A;
    cov_B([1 numSitesPerRow+1],[1 numSitesPerRow+1]) = zeros(2);
    cov_B(numSitesPerRow+2:end,numSitesPerRow+2:end) = -cov_B(numSitesPerRow+2:end,numSitesPerRow+2:end);
    
    save(paramFilepath,'cov_A','cov_B')
else
    load(paramFilepath)
end

X = normrnd(0,sqrt(cov_A));
Y = normrnd(0,sqrt(cov_B));

Z = X+1i*Y;

V = ifft2(Z);

numSitesPerRow = numSitesPerRow/scale;

Vtrue = V(1:numSitesPerRow,1:numSitesPerRow);
Vtrue = real(Vtrue);

epsilons = Vtrue(1:nSitesY,1:nSitesX);

% Make a column from epsilons. Note that the site creator will create the
% grid of coordinates and create a column list of these coordinates in the
% same order
epsilons = epsilons(:); 





